/**
 *  Copyright (C) 2010-2020 The Catrobat Team
 *  (http://developer.catrobat.org/credits)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as
 *  published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  An additional term exception under section 7 of the GNU Affero
 *  General Public License, version 3, is available at
 *  (http://developer.catrobat.org/license_additional_term)
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/.
 */

#import "AppDelegate.h"
#import "Util.h"
#import <AVFoundation/AVFoundation.h>
#import "ScenePresenterViewController.h"
#import "KeychainUserDefaultsDefines.h"
#import "CatrobatTableViewController.h"
#import "Pocket_Code-Swift.h"


void uncaughtExceptionHandler(NSException *exception)
{
    NSError(@"uncaught exception: %@", exception.description);
}

@implementation AppDelegate

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    NSSetUncaughtExceptionHandler(&uncaughtExceptionHandler);

    [self initNavigationBar];
    [ThemesHelper changeAppearance];
    
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    NSDictionary *appDefaults = [NSDictionary dictionaryWithObject:@"YES"
                                                            forKey:@"lockiphone"];
    [defaults registerDefaults:appDefaults];
    
    application.statusBarHidden = NO;
    application.statusBarStyle = UIStatusBarStyleLightContent;
    
    [self setDefaultUserDefaults: defaults];
    [defaults synchronize];

    if ([[[NSProcessInfo processInfo] arguments] containsObject: @"UITests"]) {
        UIApplication.sharedApplication.keyWindow.layer.speed = 10.0;
    }
    
    [self setupSiren];
    [self setupFirebase];

    return YES;
}

-(void)applicationDidBecomeActive:(UIApplication *)application
{
    UINavigationController *vc = (UINavigationController*)self.window.rootViewController;
    
    if ([vc.topViewController isKindOfClass:[ScenePresenterViewController class]]){
        ScenePresenterViewController* spvc = (ScenePresenterViewController*)vc.topViewController;
        [spvc resumeAction];
    }
}

- (void)applicationWillResignActive:(UIApplication *)application
{
    UINavigationController *vc = (UINavigationController*)self.window.rootViewController;
    
    if ([vc.topViewController isKindOfClass:[ScenePresenterViewController class]]){
        ScenePresenterViewController* spvc = (ScenePresenterViewController*)vc.topViewController;
        [spvc pauseAction];
    }
}

- (void)applicationDidEnterBackground:(UIApplication *)application
{
    [[AVAudioSession sharedInstance] setActive:NO error:nil];
}


- (void)applicationWillEnterForeground:(UIApplication *)application
{
    // Called as part of the transition from the background to the inactive state; here you can undo many of the changes made on entering the background.
    UINavigationController *vc = (UINavigationController*)self.window.rootViewController;
    
    if ([vc.topViewController isKindOfClass:[ScenePresenterViewController class]]){
        ScenePresenterViewController* spvc = (ScenePresenterViewController*)vc.topViewController;
        [spvc resumeAction];
    }
}

- (void)setDefaultUserDefaults:(NSUserDefaults*)defaults {
    if (![Util isPhiroActivated]) {
        [defaults setBool:NO forKey:kUsePhiroBricks];
    }
    if (![Util isArduinoActivated]) {
        [defaults setBool:NO forKey:kUseArduinoBricks];
    }
    if ([defaults valueForKey:kFirebaseSendCrashReports] == nil) {
        [defaults setBool:kFirebaseSendCrashReportsDefault forKey:kFirebaseSendCrashReports];
    }
}

- (void)initNavigationBar
{
    [UINavigationBar appearance].barTintColor = UIColor.navBar;
    [UINavigationBar appearance].tintColor = UIColor.navTint;
    [UINavigationBar appearance].barStyle = UIBarStyleBlack;
    [[UINavigationBar appearance] setTitleTextAttributes:@{NSForegroundColorAttributeName: UIColor.navText}];
    self.window.tintColor = UIColor.globalTint;
}

-(BOOL)application:(UIApplication *)app openURL:(NSURL *)url options:(NSDictionary<UIApplicationOpenURLOptionsKey,id> *)options
{
    UINavigationController* vc = (UINavigationController*)self.window.rootViewController;
    [vc popToRootViewControllerAnimated:YES];
    
    if ([vc.topViewController isKindOfClass:[CatrobatTableViewController class]]){
        CatrobatTableViewController* ctvc = (CatrobatTableViewController*)vc.topViewController;
        [ctvc addProjectFromInbox];
        return YES;
    }
    return NO;
}

@end
